@@ -2,29 +2,18 @@
 Module regrouping some functions for image processing.
 Some of the functions make use of Python Imaging Library and NumPy.
 """
-import warnings, math
-
-try:
-    from PIL import Image, ImageFilter
-    HAS_PIL = True
-except ImportError:
-    HAS_PIL = False
-
-try:
-    import numpy
-    import pygame.surfarray as surfarray
-    HAS_NUMPY = True
-except ImportError:
-    HAS_NUMPY = False
+import math
+import numpy
+from PIL import Image, ImageFilter
 
 import pygame
+import pygame.surfarray as surfarray
 from pygame.image import tostring, fromstring
 from pygame import Surface, SRCALPHA
 import pygame.gfxdraw as gfx
 
 from functools import cache
 
-
 MAX_NORM = 3*(255**2)
 
 def darken(color, factor=0.8, also_alpha=False):
@@ -150,9 +139,9 @@ def color_rect(gradient_color, size):
     gradient color can be:<br>
     *3-tuple or 4-tuple specifying the RGB or RGBA value of the color (in this case, this is a simple color);<br>
     *Tuple or list of 3-tuples or 4-tuples, plus an optional string at the end
-        indicating the orientation of the gradient : either 'h', 'v' 'h' 'v', 'r'(radial) or 'q'(square).
+        indicating the orientation of the gradient : either 'h', 'v', 'r'(radial) or 'q'(square).
         In this case, a gradient of colors is defined. Alpha component is ignored for gradients.<br>
-
+        When the orientation is 'q' (square), colors are on the form (topleft, topright, bottomleft, bottomright).<br>
     All examples below are valid:<br>
         (255,0,0) #red with alpha = 255<br>
         (255,0,0,90) #red with alpha = 90<br>
@@ -202,8 +191,8 @@ def color_gradient(colors, size, orientation):
     elif orientation == "q":
         r = pygame.Surface((2,2))
         gfx.pixel(r,0,0,colors[0])
-        gfx.pixel(r,0,1,colors[2])
         gfx.pixel(r,1,0,colors[1])
+        gfx.pixel(r,0,1,colors[2])
         gfx.pixel(r,1,1,colors[3])
     else:
         raise Exception("Orientation must be either 'h', 'v', 'r' or 'q'.")
@@ -297,10 +286,6 @@ def set_alpha_from_intensity(surface, alpha_factor, decay_mode, color):
 
 def detect_frame(surf, vacuum=(255, 255, 255)):
     """_Returns a Rect of the minimum size to contain all that is not vacuum."""
-    if not HAS_NUMPY:
-        warnings.warn("Numpy was not found on this machine.\
-            Cannot call detect_frame. Returns surface's size instead.")
-        return surf.get_size()
     vacuum = numpy.array(vacuum)
     array = pygame.surfarray.array3d(surf)
     x_found = False
@@ -496,6 +481,33 @@ def extract_frames(src, out_folder=None, size_factor=(1., 1.)):
             break
     return imgs
 
+def spritesheet_frames(src, nx, ny, line_number, colorkey=None):
+    """Return a list of pygame surfaces corresponding to a given line of the spritesheet.
+    ***Mandatory arguments***
+    <src> : filename of the spritesheet.
+    <nx> : number of sprites along x-axis.
+    <ny> : number of sprites along y-axis.
+    <line_number> : line at which the frames are picked."""
+    sheet = pygame.image.load(src)
+    w,h = sheet.get_size()
+    assert w%nx == 0 and h%ny == 0
+    frame_w = w // nx
+    frame_h = h // ny
+    frame_size = (frame_w, frame_h)
+    frames = []
+    current_rect = pygame.Rect((0,0), frame_size)
+    y = line_number * frame_h
+    x = 0
+    for i in range(nx):
+        current_rect.topleft = x,y
+        s = pygame.Surface(current_rect.size)
+        s.blit(sheet, (0,0), current_rect)
+        if colorkey:
+            s.set_colorkey(colorkey)
+        frames.append(s)
+        x += frame_w
+    return frames
+
 def draw_arrow(screen, start_coord, end_coord, arrow_color):
     """Draws an arrow on the screen from start_coord to end_coord, pointing towards end_coord."""
     angle = math.atan2(end_coord[1] - start_coord[1], end_coord[0] - start_coord[0])
@@ -829,4 +841,42 @@ def scale_image_with_constraint(img, w, h, mode, smooth=True):
 #     s.fill((255, 255, 255, alpha), special_flags=pygame.BLEND_RGBA_MIN)
 #     if n_smooth == 1:
 #         return s
-#     return pygame.transform.smoothscale(s, orig_size)
\ No newline at end of file
+#     return pygame.transform.smoothscale(s, orig_size)
+
+
+#Borrowed from pgzero
+_circle_cache = {}
+def _circlepoints(r):
+    r = int(round(r))
+    if r in _circle_cache:
+        return _circle_cache[r]
+    x, y, e = r, 0, 1 - r
+    _circle_cache[r] = points = []
+    while x >= y:
+        points.append((x, y))
+        y += 1
+        if e < 0:
+            e += 2 * y - 1
+        else:
+            x -= 1
+            e += 2 * (y - x) - 1
+    points += [(y, x) for x, y in points if x > y]
+    points += [(-x, y) for x, y in points if x]
+    points += [(x, -y) for x, y in points if y]
+    points.sort()
+    return points
+
+def render_outlined_text(text, font, gfcolor=pygame.Color('dodgerblue'),
+                         ocolor=(255, 255, 255), opx=2):
+    textsurface = font.render(text, True, gfcolor).convert_alpha()
+    w = textsurface.get_width() + 2 * opx
+    h = font.get_height()
+    osurf = pygame.Surface((w, h + 2 * opx)).convert_alpha()
+    osurf.fill((0, 0, 0, 0))
+    surf = osurf.copy()
+    osurf.blit(font.render(text, True, ocolor).convert_alpha(), (0, 0))
+    for dx, dy in _circlepoints(opx):
+        surf.blit(osurf, (dx + opx, dy + opx))
+
+    surf.blit(textsurface, (opx, opx))
+    return surf
\ No newline at end of file
